import json
import requests
import configparser

# Introduction..
# In this Tutorial we will write a simple python code that can send a message to any phone number
# You can run this application either on Terminal or in your preferred IDE.
# The application will receive 2 input parameters that will be asked for the user upon its execution
# First parameter will be the Message to be sent.
# Second parameter will be the phone that will receive the message.
# Requirements:
# Basic Programming experience with python.
# Python 3 Installed.
# requests Library Installed (http://docs.python-requests.org/en/master/).
# Generated an app_key and app_secret through the Live Link Portal
# For that, go to the portal https://livelink.sapmobileservices.com/documentation/
# Then go to Getting Started >> Step 3: Create your first APP Key


# Step 1
# SETUP ENVIROMENT VARIABLES #
# On this step we will setup all the needed Variables to make the Live Link System Work.
# base_url -> the Live Link URL that will be used to make the calls to Live Link API
# app_key -> the key generated through the Live Link Portal
# app_secret -> the secret that you received when creating a new app_key through the Live Link Portal
# default_origin -> that is the origin that you want the phone number to receive messages from.
# All this information will be used in the next functions.

config = configparser.ConfigParser()
config.read('./resources/config.ini')

base_url = config['API']['base_url']
app_key = config['API']['app_key'] # Insert your generated app key here
app_secret = config['API']['app_secret'] # Insert your generated app secret here
default_origin = config['API']['default_origin'] # Insert your default origin here

# Extra info.
# If you are Behind a corporate proxy, you need to set the proxy to send HTTP Calls
# global_proxy = {
#  'http': config['PROXY']['http'],
#  'https': config['PROXY']['https']
# }

# Step 2
# Defining Function to retrieve the oAuth2 access_token through client_credentials
def getAccessToken(key, secret, proxy=None):
    oauth_url = base_url + "/oauth/token"

    payload = {'grant_type': "client_credentials"}

    headers = {
        'content-type': "application/x-www-form-urlencoded"
    }

    if 'global_proxy' in globals(): proxy = global_proxy
    # possible to implement some logic to validate expiration time so don't request token for each  call.

    print("...... executing authorization request with key: " + key)
    result = requests.post(
        url=oauth_url,
        data=payload,
        headers=headers,
        auth=(key, secret),
        proxies=proxy
    )

    if result.status_code != 200:
        return false

    access_token = json.loads(result.text)['access_token']

    print("...... authorization request successfull access token: " + access_token)

    return access_token


# Step 3
# Function to send message with two mandatory parameters, message and destination.
# When calling this function, this will use requests library to prepare an HTTP Call
# This HTTP Call will have an Authorization Header with a Bearer access_token
# that was retrieved from the getAccessToken function.

def sendSMS(message, destination, origin=default_origin, base_url=base_url, key=app_key, secret=app_secret,
                proxy=None):
    message_url = base_url + "/v2/sms"

    payload = {
        'message': message,
        'origin': origin,
        'destination': removePlus(destination)  # LiveLink currently Supports the Phone without the + sign
    }

    headers = {
        'content-type': "application/json",
        'authorization': "Bearer " + getAccessToken(key, secret)
    }

    if 'global_proxy' in globals(): proxy = global_proxy

    print("..... sending message: " + message + ", to: " + destination)

    result = requests.post(
        url=message_url,
        data=json.dumps(payload),
        headers=headers,
        proxies=proxy
    )

    if result.status_code != 200:
        user_message = json.loads(result.text)['userMessage']
        print("..... failed to send message")
        print("..... Error: " + user_message)
        return user_message

    print("..... message sent Successfully")
    return result.text


def removePlus(phone):
    return phone.replace("+", "")


# Step 4
# Now we will write a program that receives 2 inputs
# First it will ask for the Message that you want to send
# Then it will ask for the phone number that you want to send the message to.

####################################################
#           Program Execution Start
####################################################


print("Starting message application")

text = input("What do you want to say(text message): ")
destination = input("Who do you want to send to(phone number): ")

print("sending: " + text)
print("to:      " + destination)

sendSMS(text, destination)